/*  sjm2mml.h:  header file for sjm2mml
    Takeshi H.
    copyleft  2002-2003
*/
#ifndef SJM2MML_H
#define  SJM2MML_H

#include "sjm2mml_ver.h"

/*  constants for general purposes  */
#define SAME          0    /*  for strcmp()  */

#define FNLEN      1024    /*  length of filename  */
#define STRNOTELEN   10    /*  length of pitch string ex. O4C-%128&'\0'  */
#define DRUM_CHANNEL 10    /*  drumset is on ch 10  */

#define USERCONFIGDIR ".sjm2mml"  /*  user directory that contaings config file(s)  */

/*  constants for drum def file  */
#define DEFAULT_DRUMINSTFILE "drums.def"
#define DEFAULT_ORIGINSTFILE "origmelos.def"
#define DEFAULT_ORIGDRUMINSTFILE "origdrums.def"
#define DEFLINELEN 256    /*  length of line in drum def file  */
#define INST_DRUM_COMMENT   '#'   /*  comment mark in drum def file  */

/*  in .sjm header  */
#define FORMAT_ID_STR      "Sharp JisakuMelody"
#define FORMAT_ID_STR_SIZE  0x12  /*  length of string above  */
#define VERSION_ID_STR_SIZE 0x1b  /*  length of string including version  */
#define TR_SET_LEN 4  /*  length of track setting per track  */

#define INST_DRUMSET_START  0x80  /*  1st drum instrument number  */
#define INST_DRUMS          0x3D  /*  how many drum instruments in  inst-set?  */
#define INST_ORIGINAL_INST_START (INST_DRUMSET_START + INST_DRUMS )
#define INST_ORIGINAL_INSTS 8         /*  how many original instruments in  inst-set?  */
#define INST_ORIGINAL_DRUM_START (INST_ORIGINAL_INST_START + INST_ORIGINAL_INSTS )
#define INST_ORIGINAL_DRUMS 8         /*  how many original drums in inst-set?  */

enum YesNo{Yes=0,No};
enum OkNg{OK=0,NG};

/*  exit code  */
enum ecode{Normal=0, Help, FileOpenFailure, TooManySJM, MallocError,
           DataError, IntensityError, TempoError, 
           OctaveError, PitchError, LengthError,
           TooManyTracks, TooLongString
};


/*  groups of instruments  */
enum InstrumentCode{ NormalInst, NormalDrum, 
                    OriginalInst, OriginalDrum, 
                    Other
};


/*  defaults  for original instruments  */
enum Instrument{
    Piano      =0, 
    Glocken    =9,
    ReedOrgan  =20,
    NylonGuitar=24,
    Violin     =40,
    Strings1   =48,
    Trumpet    =56,
    SopranoSax =64,

    SnareL      =168,
    BassDrumL   =171,
    FloorTomL   =174,
    ClosedHiHat =180,
    PedalHiHat  =181,
    OpenHiHat   =182,
    RideCymbal  =185,
    SplashCymbal=188
};


/*  structs  */

/*  to ease access to each part of header.  */
struct st_sjm_header {
    char version_id;
    char version_id_str[VERSION_ID_STR_SIZE];
    char flag1;
    char flag2;
    char flag3;
    unsigned char tempo;
    unsigned char num_of_tracks;
    unsigned char title_length;
};

typedef struct st_sjm_header sjm_header; 

struct st_whole_sjm {
    struct stat *bufstat;  /*  buffer for stat()  */
    char *sjmbuf;          /*  buffer for sjm data  */
    sjm_header *sh;
    unsigned int tempo;
    char *setting_of_tracks;  /*  as a pointer to beginning of settings  */
    char *title;     /*  title[]  */

    unsigned int  *notes;      /*  for notes[]  */
    unsigned char *velocity;   /*  for velocity[]  */
    unsigned char *instrument; /*  for instrument[]  */
    char *melody_top;
};

typedef struct st_whole_sjm whole_sjm; 

struct st_track_setting {
    unsigned char tr_len_low;
    unsigned char tr_len_high;
    unsigned char instrument;
    unsigned char intensity;
};

typedef struct st_track_setting track_setting; 

struct st_flags{
    enum YesNo help_exit;        /*  print help, then exit  */
    enum YesNo read_drumdef;     /*  read drums.def file  */
    enum YesNo read_origmelodef;     /*  read origmelos.def file  */
    enum YesNo read_origdrumdef;     /*  read origdrums.def file  */
    enum YesNo keep_emptytrack;  /*  keep empty track  */
    enum YesNo stdout;           /*  output is standard out  */
};

/*  whether memoly is allocated or not  */
struct st_mem_flags{
    enum YesNo bufstat;     /*  buffer for stat()  */
    enum YesNo sjmbuf;      /*  buffer for .sjm file  */
    enum YesNo notes;       /*  buffer for sjm.notes  */
    enum YesNo velocity;    /*  buffer for sjm.velocity  */
    enum YesNo instrument;  /*  buffer for sjm.instrument  */
    enum YesNo title;       /*  buffer for sjm.title  */
};

/*  functions called from outside thier residence  */
extern sjm_header *load_sjm_file(char *sjmfilename, whole_sjm *sjmp);
extern void get_header_info(whole_sjm *sjmp);
extern enum OkNg load_drum_inst(char *druminst_filename, char *druminst);
extern char *pitch_of_note( char *str_note, char onkai );
extern char *length_of_note(char *str_note, char nagasa);
extern void clean_exit(enum ecode exit_code);

/*  global variables  */
// extern struct st_whole_sjm sjm;
//extern char inst_drums[];  /*  array to convert drum inst No.  */
extern struct st_mem_flags mem_flags;

#endif  /*  ifndef SJM2MML_H  */

