/*  sjmheader.c: functions to read SJM header for sjm2mml
    Takeshi H.
    copyleft  2002-2003
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "sjm2mml.h"

/*  get header information of sjmfile, set element in struct sjm  */
    
void get_header_info(whole_sjm *sjmp)
{
    char speed;
    char tracks;
    char title_len;
    char *title_in_header;
    int  i;
    char *p;

    /*  Check the version of data  */
#ifdef DEBUG
    fprintf(stderr, "format id =%s \n",sjmp ->  sh -> version_id_str);
#endif
    if ( strncmp( sjmp ->  sh -> version_id_str, 
                  FORMAT_ID_STR, 
                  FORMAT_ID_STR_SIZE       ) != SAME ){
        fputs( "Not recognized Version or Broken Data.\n", stderr);
        clean_exit( DataError );
    }

    speed = sjmp -> sh->tempo;
#ifdef DEBUG
    fprintf(stderr, "speed =%d \n",speed);
#endif
    /*  Tempo  defined in J-SH51 User's manual (pp.194)  */
    switch ( speed ) {
        case 0x0:
            sjmp -> tempo = 150;
            break;
        case 0x1:
            sjmp -> tempo = 125;
            break;
        case 0x2:
            sjmp -> tempo = 107;
            break;
        case 0x3:
            sjmp -> tempo = 94;
            break;
        default:
            fputs( "Tempo Error.\n", stderr);
            clean_exit ( TempoError );
    }

    tracks = sjmp -> sh->num_of_tracks;
#ifdef DEBUG
    fprintf(stderr, "%d tracks\n",tracks);
#endif
    /*  Setting of tracks  */
    if ( (sjmp -> notes      = (unsigned int *)malloc(sizeof(int) * tracks) )
                    != NULL ){
        /*  Mark the malloc flag  */
        mem_flags.notes = Yes;
    } else {
#ifdef DEBUG
        fprintf(stderr, "Malloc Error for sjmp -> notes\n");
#endif
        clean_exit( MallocError ) ;
    }
    if ( (sjmp -> velocity   = (unsigned char *)malloc(sizeof(int) * tracks) )
                    != NULL ){
        /*  Mark the malloc flag  */
        mem_flags.velocity = Yes;
    } else {
#ifdef DEBUG
        fprintf(stderr, "Malloc Error for sjmp -> velocity\n");
#endif
        clean_exit( MallocError ) ;
    }
    if ( (sjmp -> instrument = (unsigned char *)malloc(sizeof(int) * tracks) ) 
                    != NULL ){
        /*  Mark the malloc flag  */
        mem_flags.instrument = Yes;
    } else {
#ifdef DEBUG
        fprintf(stderr, "Malloc Error for sjmp -> instrument\n");
#endif
        clean_exit( MallocError ) ;
    }

    /*  read in track settings  */
    sjmp -> setting_of_tracks = (char *)&(sjmp -> sh->title_length);
    sjmp -> setting_of_tracks++;
    
    for( i= 0, p = sjmp -> setting_of_tracks; i< tracks; i++) {
        sjmp -> notes[i] =  ((track_setting *)p)->tr_len_high << 8;
        sjmp -> notes[i] += ((track_setting *)p)->tr_len_low;
        sjmp -> instrument[i] = ((track_setting *)p)->instrument;

        /*  intensity  -->  velocity  */
        switch ( ((track_setting *)p)->intensity ){
            case 0x0:
                sjmp -> velocity[i] = 40;
                break; 
            case 0x1:
                sjmp -> velocity[i] = 64;
                break; 
            case 0x2:
                sjmp -> velocity[i] = 100;
                break; 
            default:
                fputs( "Invalid intensity of Track!\n", stderr);
                clean_exit( IntensityError );
        }
        p += sizeof(track_setting);
    }

    /*  read title  */
    title_len = sjmp -> sh->title_length;
#ifdef DEBUG
    fprintf(stderr, "title is %d long\n",title_len);
#endif

    if ( (sjmp -> title = (char *)malloc(title_len +1) ) != NULL ){
        /*  Mark the malloc flag  */
        mem_flags.title = Yes;
    } else {
#ifdef DEBUG
        fprintf(stderr, "Malloc Error for sjmp -> title\n");
#endif
        clean_exit( MallocError ) ;
    }

    title_in_header = (char *)&(sjmp -> sh->title_length);
    title_in_header ++;
    title_in_header += TR_SET_LEN* tracks;
    strncpy(sjmp -> title, 
            title_in_header,
            title_len );
    sjmp -> title[title_len] = '\0';  /*  put an end mark  */
#ifdef DEBUG
    fputs(sjmp -> title, stderr);
    fputs("\n", stderr);
#endif
    
#ifdef DEBUG
    for( i=0; i< tracks; i++)
        fprintf(stderr, "sjmp -> notes[%d] = %d\n",i,sjmp -> notes[i]);
#endif

    sjmp -> melody_top = title_in_header + title_len ;
}

