/*  sjmnote.c:  note conversion functions for sjm2mml.
    Takeshi H.
    copyleft  2002-2004
*/
#include <stdio.h>

#include "sjm2mml.h"


enum InstrumentCode parse_instrument( unsigned char inst )
{
    if ( inst < INST_DRUMSET_START ){
        return NormalInst;
    } else if ( INST_DRUMSET_START <= inst 
     && inst < INST_ORIGINAL_INST_START ){
        return NormalDrum;
    } else if ( INST_ORIGINAL_INST_START <= inst 
        && inst < INST_ORIGINAL_DRUM_START ){
        return OriginalInst;
    } else if ( INST_ORIGINAL_DRUM_START <= inst 
        && inst < INST_ORIGINAL_DRUM_START + INST_ORIGINAL_DRUMS ){
        return OriginalDrum;
    }

}


char *pitch_of_note(char *str_pitch, char onkai)
{
    int i;


    /*  clear string  */
    for( i=0; i < STRNOTELEN ; i++ )
        str_pitch[i]='\0';

    if (onkai == 0x0c)                  /*  rest  */
        strcpy( str_pitch, "r" );
    else {
        switch( onkai & 0xf0 ){
            case 0x30:
                /*  Octave = 2  */
                strcpy( str_pitch, "o2" );
                break;
            case 0x40:
                /*  Octave = 3  */
                strcpy( str_pitch, "o3" );
                break;
            case 0x50:
                /*  Octave = 4  */
                strcpy( str_pitch, "o4" );
                break;
            case 0x60:
                /*  Octave = 5  */
                strcpy( str_pitch, "o5" );
                break;
            case 0x70:
                /*  Octave = 6  */
                strcpy( str_pitch, "o6" );
                break;
            default:
                fputs( "Data Error: Octave is out of range.\n", stderr);
                clean_exit( OctaveError );
        }

        switch( onkai & 0x0f ){
            case 0:
                /*  note="c";  */
                strcat( str_pitch, "c" );
                break;
            case 1:
                /*  note="c+";  */
                strcat( str_pitch, "c+" );
                break;
            case 2:
                /*  note="d";  */
                strcat( str_pitch, "d" );
                break;
            case 3:
                /*  note="d+";  */
                strcat( str_pitch, "d+" );
                break;
            case 5:
                /*  note="e";  */
                strcat( str_pitch, "e" );
                break;
            case 6:
                /*  note="f";  */
                strcat( str_pitch, "f" );
                break;
            case 7:
                /*  note="f+";  */
                strcat( str_pitch, "f+" );
                break;
            case 9:
                /*  note="g";  */
                strcat( str_pitch, "g" );
                break;
            case 0xa:
                /*  note="g+";  */
                strcat( str_pitch, "g+" );
                break;
            case 0xb:
                /*  note="a";  */
                strcat( str_pitch, "a" );
                break;
            case 0xd:
                /*  note="a+";  */
                strcat( str_pitch, "a+" );
                break;
            case 0xe:
                /*  note="b";  */
                strcat( str_pitch, "b" );
                break;
            default:
                fputs( "Pitch Error\n", stderr);
                clean_exit( PitchError );
        }
    }

    return str_pitch;
}

char *length_of_note(char *str_notelen, char nagasa)
{
    int i;


    /*  clear string  */
    for( i=0; i < STRNOTELEN ; i++ )
        str_notelen[i]='\0';


    /*  length of note  "Nagasa"  */
    switch (nagasa & 0x0f){
        case 0x0e:
            strcat(str_notelen, "%8");    /*  48/4 *2 /3  */
            break;
        case 0x0d:
            strcat(str_notelen, "16");
            break;
        case 0x0b:
            strcat(str_notelen, "%16");   /*  48/2 *2 /3  */
            break;
        case 0x0a:
            strcat(str_notelen, "8");
            break;
        case 0x09:
            strcat(str_notelen, "8.");
            break;
        case 0x08:
            strcat(str_notelen, "%32");   /*  48   *2 /3  */
            break;
        case 0x07:
            strcat(str_notelen, "4");
            break;
        case 0x06:
            strcat(str_notelen, "4.");
            break;
        case 0x05:
            strcat(str_notelen, "%64");   /*  48*2 *2 /3  */
            break;
        case 0x04:
            strcat(str_notelen, "2");
            break;
        case 0x03:
            strcat(str_notelen, "2.");
            break;
        case 0x02:
            strcat(str_notelen, "%128");  /*  48*4 *2 /3  */
            break;
        case 0x01:
            strcat(str_notelen, "1");
            break;
        default:
            fputs( "Strange Length of Note\n", stderr);
            clean_exit( LengthError );
    }

    if ( (nagasa & 0x10) == 0x10 )  /*  slar  */
        strcat(str_notelen, "&");


    return str_notelen;
}
